# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    HprimDate,
    HprimForeign,
    HprimString,
    Segment,
)


class NovodiagSegmentH(HeaderSegment):
    sender_id = HprimString((5, 1))
    computer_id = HprimString((5, 2))
    instrument_id = HprimString((5, 3))
    cou_id = HprimString((5, 4))
    receiver_id = HprimString(10)
    processing_id = HprimString(12, default="P")
    version = HprimString(13, default="1394-97")
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class NovodiagSegmentP(Segment):
    patient_id = HprimString(5)
    patient_family_name = HprimString((6, 1))
    patient_given_name = HprimString((6, 2))

    orders = HprimForeign("O", "patient")


class NovodiagSegmentO(Segment):
    specimen_id = HprimString(3)
    test_id = HprimString((5, 4))
    priority = HprimString(6)
    ordered_date = HprimDate(7)

    results = HprimForeign("R", "order")


class NovodiagSegmentR(Segment):
    test_id = HprimString((3, 4))
    test_name = HprimString((3, 5))
    test_version = HprimString((3, 6))
    cartridge_lot = HprimString((3, 7))
    target_name = HprimString((3, 8))
    value = HprimString(4)
    result_status = HprimString(9)
    operator = HprimString(11)
    date_test_started = HprimDate(12)
    date_test_completed = HprimDate(13)
    cou_id = HprimString(14)


class NovodiagSegmentC(Segment):
    comment_source = HprimString(3)
    comment_text = HprimString(4)
    comment_type = HprimString(5)


class NovodiagSegmentL(Segment):
    terminaison_code = HprimString(3, default="F")


class Driver(AstmMessageV2):
    SPEC = {"code": "novodiag", "label": "Amplidiag/Novodiag ASTM de Mobidiag"}

    SEGMENT_CLASS_MAP = {
        "H": NovodiagSegmentH,
        "P": NovodiagSegmentP,
        "O": NovodiagSegmentO,
        "R": NovodiagSegmentR,
        "C": NovodiagSegmentC,
        "L": NovodiagSegmentL,
    }

    DEFAULT_SEPARATORS = "|@^&"

    OPTIONS = [
        AstmMessageV2.Option(
            "novodiag_computer_id",
            label="Id de l'émetteur pour la demande dans le champ H.5.2",
            default="Host",
        ),
        AstmMessageV2.Option(
            "novodiag_receiver_id",
            label="Id du receveur pour la demande dans le champ H.10",
            default="Novodiag",
        ),
    ]

    def __init__(self, **kwargs):
        self.computer_id = kwargs.pop("novodiag_computer_id", "Host")
        self.receiver_id = kwargs.pop("novodiag_receiver_id", "Novodiag")
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        message = NovodiagSegmentH(type_segment="H", **kwargs)
        message.end = NovodiagSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        message = self.create(
            computer_id=self.computer_id,
            receiver_id=self.receiver_id,
            date=context["now"],
        )
        for patient in worklist.get_astm_worklist(
            unique_together=["priority", "action"],
            priority={"A": "S", "S": "S", worklist.ELSE: "R"},
            action={"del": worklist.IGNORE, worklist.ELSE: ""},
        ):
            seg_p = message.patients.add(
                patient_id=patient.patient_id,
                patient_family_name=patient.last_name,
                patient_given_name=patient.first_name,
            )
            for sample, priority, action in patient.samples:
                for order in sample.orders:
                    seg_p.orders.add(
                        specimen_id=sample.sample_id,
                        test_id=order.code,
                        priority=priority,
                        ordered_date=sample.request_date,
                    )
        return message

    def decode_message(self, message):
        worklist = Worklist(context="result")
        for seg_p in message.patients:
            for seg_o in seg_p.orders:
                sample = worklist.get_or_create(
                    sample_id=seg_o.specimen_id, type="patient"
                )
                operator = None
                date = None
                for seg_r in seg_o.results:
                    comments = []
                    if seg_r.date_test_completed:
                        date = seg_r.date_test_completed
                    for seg_c in seg_r.commentaires:
                        comments.append(seg_c.comment_text)
                    if seg_r.operator:
                        operator = seg_r.operator
                    sample.results.add(
                        code=seg_r.test_id,
                        value=seg_r.value,
                        operator=operator,
                        date=date or message.date,
                        instrument=message.instrument_id,
                        comments=comments,
                    )
        return worklist
